/* RSmtli2.sl - Wavefront MTL alike for illumination model 2 - Specular
 * This shader is part of Ayam, a free 3D modeler for the RenderMan interface.
 * Copyright (c) 1998-2023 by Randolf Schultz (randolf.schultz@gmail.com)
 * All rights reserved. See License for details.
 * Parameters:
 * Ka - ambient reflectivity as color
 * Kd - diffuse reflectivity as color
 * Ks - specular reflectivity as color
 * Ns - specular exponent (1.0/roughness in RenderMan terms)
 * d  - dissolve, i.e. color independent opacity:
 *      1 - off, opaque
 *      0 - transparent
 *      - - transparent (with '-halo' option, creates opacity effect that is
 *          "dependent on the surface orientation relative to the viewer")
 * bm - bump multiplier
 * Ka_map   - multiplies Ka with colors from a texture file
 * Kd_map   - multiplies Kd with colors from a texture file
 * Ks_map   - multiplies Ks with colors from a texture file
 * Ns_map   - multiplies Ns with float from a texture file
 * d_map    - multiplies d with float from a texture file
 * bump_map - distort normal with value from a texture file
 */
surface
RSmtli2(color Ka = 1, Kd = .5, Ks = .5;
	float Ns = 1, d = 1, bm = 1;
	string Ka_map = "", Kd_map = "", Ks_map = "", Ns_map = "", d_map = "",
	       bump_map = "";)
{
  normal Nf;
  vector V;
  color Kam = Ka, Kdm = Kd, Ksm = Ks;
  float Nsm = Ns, dm = abs(d);

  if(Ka_map != "")
    Kam *= color texture(Ka_map);

  if(Kd_map != "")
    Kdm *= color texture(Kd_map);

  if(Ks_map != "")
    Ksm *= color texture(Ks_map);

  if(Ns_map != "")
    Nsm *= float texture(Ns_map);

  if(d_map != "")
    dm *= float texture(d_map);

  if(bump_map != "")
    N = calculatenormal(P + bm * (float texture(bump_map)) * normalize(N));

  Nf = faceforward(normalize(N), I);
  V = -normalize(I);

  if((d < 0) || (d == -0))
    Oi = mix(dm, 1.0, min(acos(normalize(Nf).V)/1.5708,1));
  else
    Oi = dm;

  Ci = Oi * ((Kam*ambient() + Kdm*diffuse(Nf)) +
              Ksm*specular(Nf, V, 1.0/Nsm));
}
